@@ -1,10 +1,15 @@
 ## make Rcmd check happy
-utils::globalVariables(c("Label"))
+utils::globalVariables(c("Label", "variable"))
 
 #' Calculate Marginal Effects from 'brms' Models
 #'
 #' This function is designed to help calculate marginal effects
 #' including average marginal effects (AMEs) from \code{brms} models.
+#' Arguments are labeled as \emph{required} when it is required that the
+#' user directly specify the argument. Arguments are labeled as
+#' \emph{optional} when either the argument is optional or there are
+#' sensible default values so that users do not typically need to specify
+#' the argument.
 #'
 #' The main parts required for the function are a fitted model object,
 #' (via the \code{object} argument) a dataset to be used for prediction,
@@ -60,46 +65,62 @@ utils::globalVariables(c("Label"))
 #' \code{at} or \code{add} and the contrast matrix, other types of estimates
 #' averaged or weighting results in specific ways are also possible.
 #'
-#' @param object A fitted \code{brms} model object. Required.
-#' @param at An optional object inheriting from data frame indicating
+#' @param object A \emph{required} argument specifying a fitted \code{brms} model object.
+#' @param at An \emph{optional} argument (but note, either \code{at} or \code{add} are
+#'   \emph{required}) specifying an object inheriting from data frame indicating
 #'   the values to hold specific variables at when calculating average
 #'   predictions. This is intended for AMEs from categorical variables.
-#'   Currently only one of \code{at} or \code{add} can be specified.
-#' @param add An optional object inheriting from data frame indicating
+#' @param wat An \emph{optional} list with named elements including one element named,
+#'   \dQuote{ID} with a single character string, the name of the variable
+#'   in the model frame that is the ID variable. Additionally,
+#'   there should be one or more named elements, named after variables
+#'   in the model (and specified in the \code{at} argument), that
+#'   contain a \code{data.table} or \code{data.frame} with three
+#'   variables: (1) the ID variable giving IDs, (2) the values
+#'   specified for the variable in the \code{at} argument, and
+#'   (3) the actual values to be substituted for each ID.
+#'   \code{wat} cannot be non null unless \code{at} also is non null.
+#' @param add An \emph{optional} argument (but note, either \code{at} or \code{add} are
+#'   \emph{required}) specifying an object inheriting from data frame indicating
 #'   the values to add to specific variables at when calculating average
 #'   predictions. This is intended for AMEs for continuous variables.
-#'   Currently only one of \code{at} or \code{add} can be specified.
-#' @param newdata An object inheriting from data frame indicating
-#'   the baseline values to use for predictions and AMEs.
-#'   Defaults to be the model frame.
-#' @param CI A numeric value specifying the width of the credible interval.
-#'   Defaults to \code{0.99}.
-#' @param CIType A character string specifying the type of credible interval
-#'   (e.g., highest density interval). It is passed down to
+#' @param newdata An \emph{optional} argument specifying an object inheriting
+#'   from data frame indicating the baseline values to use for predictions and AMEs.
+#'   It uses a sensible default: the model frame from the \code{brms}
+#'   model object passed on the \code{object} argument.
+#' @param CI An \emph{optional} argument with a numeric value specifying the width
+#'   of the credible interval. Defaults to \code{0.99}. This default is arbitrary,
+#'   but is purposefully higher than the common \code{0.95} to encourage science
+#'   with greater acknowledgment of uncertainty or larger sample sizes (ideally).
+#' @param CIType An \emph{optional} argument, a character string specifying the
+#'   type of credible interval (e.g., highest density interval). It is passed down to
 #'   \code{\link{bsummary}} which in turn passes it to
 #'   \code{\link[bayestestR]{ci}}. Defaults to \dQuote{HDI}.
-#' @param contrasts An optional contrast matrix. The posterior predictions matrix
+#' @param contrasts An \emph{optional} argument specifying a contrast matrix.
+#'   The posterior predictions matrix
 #'   is post multiplied by the contrast matrix, so they must be conformable.
 #'   The posterior predictions matrix has a separate column for each row in the
 #'   \code{at} or \code{add} object, so the contrast matrix should have the same
 #'   number of rows. It can have multiple columns, if you desire multiple specific
 #'   contrasts.
-#' @param ROPE Either left as \code{NULL}, the default, or a numeric vector of
-#'   length 2, specifying the lower and upper thresholds for the
+#' @param ROPE An \emph{optional} argument, that can either be left as \code{NULL},
+#'   the default, or a numeric vector of length 2, specifying the
+#'   lower and upper thresholds for the
 #'   Region of Practical Equivalence (ROPE).
-#' @param MID Either left as \code{NULL}, the default, or a numeric vector of
-#'   length 2, specifying the lower and upper thresholds for a
+#' @param MID An \emph{optional} argument, that can either left as \code{NULL},
+#'   the default, or a numeric vector of length 2, specifying the
+#'   lower and upper thresholds for a
 #'   Minimally Important Difference (MID). Unlike the ROPE, percentages for
 #'   the MID are calculated as at or exceeding the bounds specified by this
 #'   argument, whereas the ROPE is the percentage of the posterior at or inside
 #'   the bounds specified.
-#' @param subset A character string that is a valid \code{R} expression
-#'   used to subset the dataset passed in \code{newdata},
+#' @param subset An \emph{optional} argument, a character string that is a
+#'   valid \code{R} expression used to subset the dataset passed in \code{newdata},
 #'   prior to analysis. Defaults to \code{NULL}.
-#' @param dpar Parameter passed on the \code{dpar}
-#'   argument of \code{fitted()} in brms. Defaults to \code{NULL}
+#' @param dpar An \emph{optional} argument giving the parameter passed on to the \code{dpar}
+#'   argument of \code{fitted()} in brms. Defaults to \code{NULL},
 #'   indicating the mean or location parameter typically.
-#' @param seed Argument that controls whether (and if so what) random seed
+#' @param seed An \emph{optional} argument that controls whether (and if so what) random seed
 #'   to use. This does not matter when using fixed effects only. However,
 #'   when using Monte Carlo integration to integrate out random effects from
 #'   mixed effects models, it is critical if you are looking at a continuous
@@ -121,14 +142,17 @@ utils::globalVariables(c("Label"))
 #'   This would be fine, for instance, when only using fixed effects,
 #'   or if you know what you are doing and intend that behavior when
 #'   integrating out random effects.
-#' @param ... Additional arguments passed on to \code{\link{prediction}}.
-#'   In particular, the \code{effects} argument of [prediction()] is
-#'   important for mixed effects models to control how random effects
+#' @param verbose An \emph{optional} argument, a logical value whether to print
+#'   more verbose messages. Defaults to \code{FALSE} which is quieter. Set to
+#'   \code{TRUE} for more messages to be printed where relevant.
+#' @param ... An \emph{optional} argument, additional arguments passed on to
+#'   \code{\link{prediction}}. In particular, the \code{effects} argument of [prediction()]
+#'   is important for mixed effects models to control how random effects
 #'   are treated in the predictions, which subsequently changes the
 #'   marginal effect estimates.
 #' @importFrom stats model.frame runif
-#' @importFrom data.table as.data.table copy :=
-#' @importFrom methods missingArg
+#' @importFrom data.table as.data.table is.data.table copy :=
+#' @importFrom extraoperators %gele% %nin%
 #' @return A list with four elements.
 #' \itemize{
 #'   \item{\code{Posterior}}{Posterior distribution of all predictions. These predictions default to fixed effects only, but by specifying options to [prediction()] they can include random effects or be predictions integrating out random effects.}
@@ -202,7 +226,7 @@ utils::globalVariables(c("Label"))
 #' if (FALSE) {
 #'   library(lme4)
 #'   data(sleepstudy)
-#'   fit <- brms::brm(Reaction ~ 1 + Days + (1+ Days | Subject),
+#'   fit <- brms::brm(Reaction ~ 1 + Days + (1 + Days | Subject),
 #'              data = sleepstudy,
 #'              cores = 4)
 #'
@@ -219,25 +243,35 @@ utils::globalVariables(c("Label"))
 #'   tmp$ContrastSummary
 #'   }
 #' }
-brmsmargins <- function(object, at = NULL, add = NULL, newdata = model.frame(object),
+brmsmargins <- function(object, at = NULL, wat = NULL, add = NULL, newdata = model.frame(object),
                         CI = .99, CIType = "HDI", contrasts = NULL,
                         ROPE = NULL, MID = NULL,
-                        subset = NULL, dpar = NULL, seed, ...) {
+                        subset = NULL, dpar = NULL, seed, verbose = FALSE,
+                       ...) {
+  if (isTRUE(missing(object))) {
+    stop(paste(
+      "'object' is a required argument and cannot be missing;",
+      "  it should be a saved model fit from brms. For example:",
+      "  m <- brm(y ~ x, data = yourdata)",
+      "  See ?brmsmargins or the website articles (vignettes) for details.",
+      "  https://joshuawiley.com/brmsmargins/", sep = "\n"))
+  }
   .assertbrmsfit(object)
+
   chknewdata <- .checktab(newdata)
   if (isTRUE(nzchar(chknewdata))) {
     stop(paste0("newdata: ", chknewdata))
   }
   newdata <- copy(as.data.table(newdata))
 
-  if (isFALSE(missingArg(seed))) {
+  if (isFALSE(missing(seed))) {
     if (isFALSE(is.null(seed))) {
       stopifnot(
         identical(length(seed), 1L) ||
           identical(length(seed), nrow(at)) ||
           identical(length(seed), nrow(add)))
     }
-  } else if (isTRUE(missingArg(seed))) {
+  } else if (isTRUE(missing(seed))) {
     ## create a random seed somewhere between +/- 1e7
     seed <- ceiling(runif(1, -1e7, 1e7))
   }
@@ -284,11 +318,73 @@ brmsmargins <- function(object, at = NULL, add = NULL, newdata = model.frame(obj
                sep = "\n"))
   }
 
+  # error if missing both at and add
+  if (isTRUE(is.null(at)) && isTRUE(is.null(add))) {
+    stop(paste("You must specify either 'at' or 'add'",
+               "See ?brmsmargins or vignettes for help.",
+               sep = "\n"))
+  }
+
+  if (isFALSE(is.null(add)) && isTRUE(is.null(contrasts))) {
+    if (isTRUE(verbose)) {
+      message(paste(
+        "It is unusual to specify 'add' without 'contrasts'.",
+        "Without 'contrasts', only marginal predictions will be generated.",
+        "If predictions are desired, consider using prediction() directly.",
+        "To suppress this message, set 'verbose = FALSE'", sep = "\n"))
+    }
+  }
+
+  if (isFALSE(CI %gele% c(0, 1))) {
+    stop(paste(
+      sprintf("'CI' is %s", as.character(CI)),
+      "'CI' should specify the desired credible interval as a numeric value in (0, 1)",
+      "See ?bayestestR::ci for details",
+      sep = "\n"))
+  }
+
+  if (isFALSE(CIType %in% c("HDI", "ETI", "BCI", "SI"))) {
+    stop(paste(
+      sprintf("'CIType' is %s", as.character(CIType)),
+      "'CIType' should be one of 'HDI' (default), 'ETI', 'BCI', or 'SI'",
+      "See ?bayestestR::ci for details",
+      sep = "\n"))
+  }
+
+  if (isFALSE(is.null(wat))) {
+    if (isTRUE(is.null(at))) {
+      stop("If 'wat' is specified, 'at' also must be specified.")
+    }
+
+    test.wat.type <- isTRUE(is.list(wat))
+    test.wat.id <- isTRUE("ID" %in% names(wat))
+
+    if (isFALSE(test.wat.type) || isFALSE(test.wat.id)) {
+      stop(paste(
+        "'wat' should be a list with named elements",
+        "including 'ID' giving the name of the ID variable and",
+        "separate elements (each a data.frame or data.table) containing the IDs,",
+        "the values specified in 'at' and the true values to use by ID.",
+        sep = "\n"))
+    }
+  }
+
   if (isFALSE(is.null(at))) {
     out <- vector("list", nrow(at))
     for (i in seq_len(nrow(at))) {
       for (v in names(at)) {
-        newdata[, (v) := at[i, get(v)]]
+        fill_in_value <- at[i, get(v)]
+        if (isTRUE(is.null(wat) || v %nin% names(wat))) {
+          newdata[, (v) := fill_in_value]
+        } else {
+          if (isFALSE(is.data.table(wat[[v]]))) {
+            wat[[v]] <- as.data.table(wat[[v]])
+          }
+          for (useid in unique(newdata[[wat$ID]])) {
+            newdata[get(wat$ID) == useid,
+            (v) := wat[[v]][variable == fill_in_value & get(wat$ID) == useid, value]]
+          }
+        }
       }
       if (isFALSE(is.null(seed))) {
         if (isTRUE(length(seed) > 1)) {
